;===============================================================================
;INCLUDE FILE: - LCD_ROUTINES.INC
;
;PURPOSE: - This file is included to supply the following functionality:
;
; 1. INITIALIZE LCD 
; 2. CHECK LCD BUSY FLAG 
; 3. SEND A COMMAND TO THE LCD
; 4. SEND A CHARACTER TO THE LCD.
; 5. MOVE TO 1ST ROW, FIRST COLUMN.
; 6. MOVE TO 1ST ROW, FIRST COLUMN.
; 7. MOVE TO 1ST ROW, BUT TO A COLUMN OF YOUR CHOOSING -> AT W
; 8. MOVE TO 2ND ROW, BUT TO A COLUMN OF YOUR CHOOSING -> AT W.
; 9. TURN ON DISPLAY TURN ON CURSOR.
; 10. TURN ON DISPLAY TURN OFF CURSOR.
; 11. TURN OFF DISPLAY.
;
;PIN CONNECTIONS: -
;
;	LCD (Hitachi)  
;                                 ________  
;    		                 VSS |1       |   
;                            VDD |2       |   
;                 CONTRAST   VEE |3       |   
;                 RC1------  RS  |4       |   
;                 RC2------  R/W |5       |   
;                 RC3------  E   |6       |   
;                       (Unused) |7       |                   
;                       (Unused) |8       |   
;                       (Unused) |9       |   
;                       (Unused) |10      |  
;                 RC4------  DB4 |11      |    
;                 RC5------  DB5 |12      |   
;                 RC6------  DB6 |13      |      
;                 RC7------  DB7 |14      |     
;          BACKLIGHT POWER   VDD |15      |     
;                   GROUND   VSS |16      |   
;                                 --------  
;
;===============================================================================  
; 1. INITIALIZE LCD 
;
; PURPOSE: - Initialises the LCD before anything else. 
;===============================================================================  
LCD_INIT
	clrf	PORTC		;CLEAR PORTC. ALSO SETS RS, R/W AND E LOW.
	 
	call	LCD_BUSY
	movlw	H'20'
	call	LCD_COMMAND

						; NOTE: - Having done this we now need 
						; to send our 8 bit bytes in TWO stages. 
						; We NOW do this via our CMND2LCD 
						; routine sending XXXX of XXXXYYYY first 
						; followed by YYYY second. 

						; now follow the commands to set the LCD 
						; display lines, the cursor, the blink and 
						; which way the cursor moves (to right). 

	movlw	H'28'		;DB5=1 -> FUNCTION. DB3=1 -> 2 LINE LCD.
	call	LCD_COMMAND	;SEND 00101000 IN THE FORM XXXX YYYY.

	movlw	H'06'		;DB3=1 -> CHAR ENTRY. DB2=1 -> INCREMENT.
	call	LCD_COMMAND	;SEND 00000110 IN THE FORM OF XXXX YYYY.

	movlw	H'0C'		;DB4=1 -> DISPLAY. DB3=1 -> DISPLAY ON.
	call	LCD_COMMAND	;SEND 00001100 IN THE FORM OF XXXX YYYY.

	movlw	H'01'		;CLEAR DISPLAY.
	call	LCD_COMMAND	;SEND 00000001 IN THE FORM OF XXXX YYYY.

	return

;=============================================================================== 
; 2. CHECK LCD BUSY FLAG 
;
;PURPOSE: - Checks the busy flag on the LCD. (QUICKER THAN WAITING).  
;
;To read an LCD busy flag three things must be set.  
;RS == 0 
;R/W == 1 
;DB7 -> RC7 Must be set as INPUT.  
;
;We FIRST Switch to BANK1 so it can change the PIN associated with DB7   
;(in my case its RC7) on the LCD to be an INPUT.  
;
;NOTE: - We always read in two nibbles at a time so in our instance we are   
;only interested in the HIGH nibble. (contains DB7 and we want to test for 
;DB7 -> high).  
;
;Finally we read in PORTC and save the high nibble into W for later. Next we  
;read in the lower nibble.  
;We check for DB7 to be clear... if it is we return..   
;===============================================================================  
LCD_BUSY 
	movwf	W_TEMP 		;COPY W TO TEMP REGISTER
			
	bcf		STATUS,RP1	;FOR BANK 1 RP1=0 AND RP0=1.
	bsf		STATUS,RP0	;SELECT BANK 1.
	
	movlw	B'10000000'
	movwf	TRISC		;SET PINS RC0-6 TO BE OUTPUTS. RC7 INPUT.

	bcf		STATUS,RP1	;FOR BANK 0 RP1=0 AND RP0=0.
	bcf		STATUS,RP0	;SELECT BANK 1.

	bcf		PORTC,LCD_RS;SET RS TO BE "0" - COMMAND MODE.
	bsf		PORTC,LCD_RW;SET R/W TO BE "1" - READ MODE.
	bsf		PORTC,LCD_E	;SET LCD_E = HIGH.

	movf	PORTC,W		;READ PORTC INTO W.
	movwf	LCD_READ	;SAVE THIS READ FOR LATER TESTING.

	bcf		PORTC,LCD_E	;SET LCD_E = LOW.

	bsf		PORTC,LCD_E	;SET LCD_E = HIGH. (THIS IS THE DUMMY READ
						; FOR THE LOWER NIBBLE).			 
	nop					;WAIT A WHILE...

	bcf		PORTC,LCD_E	;SET LCD_E = LOW.

	btfsc	LCD_READ,LCDBUSY ;RB7 -> RC7 SO CHECK THAT THIS IS CLEAR.
	goto	LCD_BUSY	;RB7 IS STILL HIGH SO LCD STILL BUSY.

	bcf		PORTC,LCD_RW;SET R/W TO BE "1" - BACK TO WRITE MODE.

	bcf		STATUS,RP1	;FOR BANK 1 RP1=0 AND RP0=1.
	bsf		STATUS,RP0	;SELECT BANK 1.

	movlw	B'00000000'
	movwf	TRISC		;SET PINS RC0-7 BACK TO BEING ALL OUTPUTS.

	bcf		STATUS,RP1	;FOR BANK 0 RP1=0 AND RP0=0.
	bcf		STATUS,RP0	;SELECT BANK 1.

	swapf 	W_TEMP,W 	;Swap W_TEMP into W
	return

;===============================================================================  
; 3. SEND A COMMAND TO THE LCD
;
;PURPOSE: - Send commands to the LCD.
;
;This is done via breaking up the byte (eight bits) into two four bit  
;nibbles. Always send the high nibble first and then the lower nibble.  
;So all commands to the LCD are in the the form of TWO LINES.  
;===============================================================================  
LCD_COMMAND
	movwf	LCD_CHAR	;FIRST THE UPPER NIBBLE...
	andlw	H'F0'		;SET UP MASK.
	movwf	PORTC
	bcf		PORTC,LCD_RW;SET LCD TO READ MODE (RW=0)
	bcf		PORTC,LCD_RS;SET LCD TO COMMAND MODE (RS=0)
	call	PULSE_E

						;NOW THE LOWER NIBBLE... 

	swapf	LCD_CHAR,W	;SWAP NIBBLES AND PLACE BACK INTO W.
	andlw	H'F0'		;SET UP MASK.
	movwf	PORTC
	bcf		PORTC,LCD_RW;SET LCD TO READ MODE (RW=0)
	bcf		PORTC,LCD_RS;SET LCD TO COMMAND MODE (RS=0)
	call	PULSE_E

	call	LCD_BUSY

	retlw	H'00'		;RETURN WITH "0" IN W.

;===============================================================================  
; 4. CONVERT A BINARY NUMBER TO AN ASCII CHARACTER for LCD.
;
;PURPOSE: - Convert a standard binary number less than 9 to an ascii character.  
;
;This is done by simply "OR"ing the number in "W" with H'30' as having H'30' in
;the upper nibble converts the normal number to a binary number. It then places
;it back into "W".
;
;From here it the program logic flows straight through to LCD_DISP_CHAR.
;===============================================================================  
LCD_PUT_ASC
	IORLW	H'30'		;OR THE FACTOR TO GET  THE ASCII CODES
	
;===============================================================================  
; 5. SEND A CHARACTER TO THE LCD.
;
;PURPOSE: - Send characters to the LCD.  
;
;This is done via breaking up the byte (eight bits) into two four bit  
;nibbles. Always send the high nibble first and then the lower nibble.  
;So all characters sent to the LCD are in the the form of TWO LINES.  
;===============================================================================  
LCD_DISP_CHAR
	movwf	LCD_CHAR	;FIRST THE UPPPER NIBBLE...
	andlw	H'F0'		;SET UP MASK.
	movwf	PORTC
	bcf		PORTC,LCD_RW;SET LCD TO READ MODE (RW=0)
	bsf		PORTC,LCD_RS;SET LCD TO COMMAND MODE (RS=1)
	call	PULSE_E

						;NOW THE LOWER NIBBLE... 
									 
	swapf	LCD_CHAR,W	;SWAP NIBBLES AND PLACE BACK INTO W.
	andlw	H'F0'		;SET UP MASK.
	movwf	PORTC
	bcf		PORTC,LCD_RW;SET LCD TO READ MODE (RW=0)
	bsf		PORTC,LCD_RS;SET LCD TO COMMAND MODE (RS=1)
	call	PULSE_E

	call	LCD_BUSY

	retlw	H'00'		;RETURN WITH "0" IN W.

PULSE_E	
	bsf		PORTC,LCD_E	;PULSE THE E LINE HIGH.
	nop					;WAIT.
	bcf		PORTC,LCD_E	;...AND DROP THE LINE AGAIN.

	retlw	H'00'		;RETURN WITH "0" IN W.



;===============================================================================  
; 6. MOVE TO 1ST ROW, FIRST COLUMN.
;
;PURPOSE: - Set the LCD to the first row/first column. 
;
;When you want the LCD to start at the first row and the first column.
;===============================================================================  
LCD_LINE1
	movlw	0x80	;MOVE TO THE FIRST ROW/FIRST COLUMN.
	call	LCD_COMMAND
	retlw	0x00

;===============================================================================  
; 7. MOVE TO 1ST ROW, FIRST COLUMN.
;
;PURPOSE: - Set the LCD to the second row/first column. 
;
;When you want the LCD to start at the second row and the first column.
;===============================================================================  
LCD_LINE2
	movlw	0xc0	;MOVE TO THE SECOND ROW/FIRST COLUMN.
	call	LCD_COMMAND
	retlw	0x00

;===============================================================================  
; 8. MOVE TO 1ST ROW, BUT TO A COLUMN OF YOUR CHOOSING -> AT W
;
;PURPOSE: - Set the LCD to the first row/column at -> W. 
;
;When you want the LCD to start on the first row but want it to start in a 
;column other than the first column then you load the column number into W
;and this routine will start on the first row and in that column.
;===============================================================================  
LCD_LINE1_W	
	addlw	0x80	;MOVE TO THE FIRST ROW AND A COLUMN OF YOUR CHOOSEING -> W.
	call	LCD_COMMAND
	retlw	0x00

;===============================================================================  
; 9. MOVE TO 2ND ROW, BUT TO A COLUMN OF YOUR CHOOSING -> AT W.
;
;PURPOSE: - Set the LCD to the second row/column at -> W. 
;
;When you want the LCD to start on the second row but want it to start in a 
;column other than the first column then you load the column number into W
;and this routine will start on the second row and in that column.
;===============================================================================  
LCD_LINE2_W	
	addlw	0xc0	;MOVE TO THE SECOND ROW AND A COLUMN OF YOUR CHOOSEING -> W.
	call	LCD_COMMAND
	retlw	0x00

;===============================================================================  
; 10. TURN ON DISPLAY TURN ON CURSOR.
;
;PURPOSE: - Set the LCD to turn on the display and the cursor.
;
;This routine turns on the display and also turns on the cursor.
;===============================================================================  
LCD_DIS_ON_CUR_ON
	movlw	0x0d	;SET THE DISPLAY ON AND ALSO TURN ON THE CURSOR.
	call	LCD_COMMAND
	retlw	0x00

;===============================================================================  
; 11. TURN ON DISPLAY TURN OFF CURSOR.
;
;PURPOSE: - Set the LCD to turn on the display but turns off the cursor.
;
;This routine turns on the display but turns off the cursor.
;===============================================================================  
LCD_DIS_ON_CUR_OFF	
	movlw	0x0c	;SET THE DISPLAY ON AND BUT TURN OFF THE CURSOR.
	call	LCD_COMMAND
	retlw	0x00

;===============================================================================  
; 12. DECREMENT CURSOR POSITION.
;
;PURPOSE: - Set the LCD to reverse the direction of writing.
;
;This routine makes the LCD read from right to left. This is required as we are
;writing numbers as you would like a calculator. From right to left or at least
;every time you enter a number it rewrites from right to left.
;===============================================================================  
LCD_DEC_CUR	
	movlw	0x04	;DECREMENT THE CURSOR POSITION.
	call	LCD_COMMAND
	retlw	0x00

;===============================================================================  
; 13. INCREMENT CURSOR POSITION.
;
;PURPOSE: - Set the LCD to reverse the direction of writing.
;
;This routine makes the LCD read from right to left. This is required as we are
;writing numbers as you would like a calculator. From right to left or at least
;every time you enter a number it rewrites from right to left.
;===============================================================================  
LCD_INC_CUR	
	movlw	0x06	;DECREMENT THE CURSOR POSITION.
	call	LCD_COMMAND
	retlw	0x00
	
;===============================================================================  
; 14. TURN OFF DISPLAY.
;
;PURPOSE: - Turns off the display.
;
;This routine turns off the display - makes it blank and then turns it on again.
;===============================================================================  
LCD_CLEAR		
	movlw	0x01	;Clear display
	call	LCD_COMMAND
	retlw	0x00


